package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IAntiStaticCheckService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesAntiStaticCheck;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.persistence.EntityManager;

/**
 * @author yumingxing
 * @version 1.0
 * @date 2021/2/5 15:55
 **/
@RestController
@Api(tags = "防静电手腕点检")
@RequestMapping(CommonConstWords.BASE_URL_MES + "/anti-static-check")
public class AntiStaticCheckController {

    @Autowired
    private IAntiStaticCheckService mesAntiStaticCheckService;

    @Resource(name="redisRes")
    private ImppRedis redisRes;
    @Autowired
    private EntityManager entityManager;

    /**
     * 插入
     *
     * @param antiStaticCheck mes自动
     * @return {@link ResultBean}
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增")
    public ResultBean insert(@RequestBody MesAntiStaticCheck antiStaticCheck) {
        try {
            //条件验证
            ValidatorBean.beginValid(antiStaticCheck)
                    .notNull("workCellCode", antiStaticCheck.getWorkCellCode())
                    .notNull("workCenterCode", antiStaticCheck.getWorkCenterCode())
            ;
            return ResultBean.success("新增成功").setResultObject(mesAntiStaticCheckService.insert(antiStaticCheck))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 修改单据规则
     *
     * @param antiStaticCheck mes部分自动降落
     * @return {@link ResultBean}
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改")
    public ResultBean update(@RequestBody MesAntiStaticCheck antiStaticCheck) {
        try {
            //条件验证
            ValidatorBean.beginValid(antiStaticCheck)
                    .notNull("workCellCode", antiStaticCheck.getWorkCellCode())
                    .notNull("workCenterCode", antiStaticCheck.getWorkCenterCode());
            return ResultBean.success("修改成功").setResultObject(mesAntiStaticCheckService.update(antiStaticCheck))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据ID启用或禁用单据规则
     * @param id
     * @param status
     * @return
     */
    @PutMapping(value = "/status-update")
    @ApiOperation(value = "根据ID启用或禁用")
    public ResultBean updateOrderIsValidById(long id, Integer status) {
        try {
            return ResultBean.success("操作成功").setResultObject(mesAntiStaticCheckService.updateIsValidById(id,status))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询分页
     *
     * @param antiStaticCheck mes自动
     * @param pager             寻呼机
     * @return {@link ResultBean}
     */
    @GetMapping(value = "/query")
    @ApiOperation(value = "查询分页")
    public ResultBean queryOrderRule(MesAntiStaticCheck antiStaticCheck, Pager pager) {
        try {
            return ResultBean.success("查询成功").setListPager(mesAntiStaticCheckService.queryByPager(antiStaticCheck, pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
