package cn.estsh.i3plus.ext.mes.apiservice.controller.base.jx;

import cn.estsh.i3plus.ext.mes.api.base.jx.ITorqueManageService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProcessTorqueManage;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesProcessTorqueManageTaskDetail;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Author: xinwang.yi
 * @CreateDate: 2021/01/18 13:33 下午
 * @Description:
 **/
@RestController
@Api(tags = "扭矩管理")
@RequestMapping(CommonConstWords.BASE_URL_MES + "/mes-process-torque-manage")
public class TorqueManageController {
    public static final Logger LOGGER = LoggerFactory.getLogger(TorqueManageController.class);

    @Autowired
    private ITorqueManageService torqueManageService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "添加MES_工序扭矩管理项目信息")
    public ResultBean insertMesProcessTorqueManage(MesProcessTorqueManage mesProcessTorqueManage) {
        try {
            //条件验证
            ValidatorBean.beginValid(mesProcessTorqueManage)
                    .notNull(MesExtConstWords.PART_NO,mesProcessTorqueManage.getPartNo())
                    .notNull(MesExtConstWords.PROCESS_CODE,mesProcessTorqueManage.getProcessCode())
                    .notNull(MesExtConstWords.FASTENING_POSITION, mesProcessTorqueManage.getFasteningPosition());
            mesProcessTorqueManage.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            torqueManageService.insertMesProcessTorqueManage(mesProcessTorqueManage);
            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException imppException) {
            LOGGER.error(imppException.getErrorMsg() + "：{}", imppException.getErrorDetail(), imppException);
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "更新MES_工序扭矩管理项目")
    public ResultBean updateMesProcessTorqueManage(MesProcessTorqueManage mesProcessTorqueManage) {
        try {
            //条件验证
            ValidatorBean.beginValid(mesProcessTorqueManage)
                    .notNull(MesExtConstWords.PART_NO,mesProcessTorqueManage.getPartNo())
                    .notNull(MesExtConstWords.PROCESS_CODE,mesProcessTorqueManage.getProcessCode())
                    .notNull(MesExtConstWords.FASTENING_POSITION, mesProcessTorqueManage.getFasteningPosition());

            if (mesProcessTorqueManage.getId() == null || mesProcessTorqueManage.getId().longValue() < 1) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要操作的资源。")
                        .build();
            }

            torqueManageService.updateMesProcessTorqueManage(mesProcessTorqueManage, AuthUtil.getSessionUser().getUserName());

            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/query")
    @ApiOperation(value = "按条件分页查询MES_工序扭矩管理项目信息", notes = "按条件分页查询MES_工序扭矩管理项目信息")
    public ResultBean queryMesProcessTorqueManageByPager(MesProcessTorqueManage mesProcessTorqueManage, Pager pager) {
        try {
            mesProcessTorqueManage.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success().setListPager(torqueManageService.queryMesProcessTorqueManageByPager(mesProcessTorqueManage, pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/update-status-batch")
    @ApiOperation(value = "批量禁用/启用详情", notes = "批量禁用/启用详情")
    public ResultBean updateMesProcessTorqueManageStatus(String[] idsStr, int status) {
        try {
            ConvertBean.modelSafeArrayNumber(idsStr, true);
            //登陆用户
            SessionUser user = AuthUtil.getSessionUser();

            torqueManageService.updateMesProcessTorqueManageStatus(idsStr, status, user.getUserName());
            return ResultBean.success("修改成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/detail/report-query")
    @ApiOperation(value = "扭矩管理查询报表", notes = "扭矩管理查询报表")
    public ResultBean queryTorqueManageTaskDetailReportByPager(MesProcessTorqueManageTaskDetail torqueManageTaskDetail, Pager pager) {
        try {
            torqueManageTaskDetail.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("查询成功").setListPager(torqueManageService.queryTorqueManageTaskDetailReportByPager(torqueManageTaskDetail, pager))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/detail/confirm")
    @ApiOperation(value = "点击扭矩管理明细报表中的班组确认按钮")
    public ResultBean updateMesQManageTaskDetailById(@RequestBody List<MesProcessTorqueManageTaskDetail> torqueManageTaskDetails) {
        try {
            torqueManageService.doConfirmTorqueManageList(torqueManageTaskDetails, AuthUtil.getSessionUser().getUserName());

            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
