package cn.estsh.i3plus.ext.mes.apiservice.controller.busi.bu3;

import cn.estsh.i3plus.ext.mes.api.busi.bu3.ISxLossTimeMonthlyService;
import cn.estsh.i3plus.ext.mes.pojo.model.bu3.SxLossTimeMonthlyModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.base.controller.MesBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;

@RestController
@RequestMapping(CommonConstWords.BASE_URL_MES)
@Api(tags = "月度非作业汇总（损失时间）报表控制层")
public class SxLossTimeMonthlyController extends MesBaseController {

    @Autowired
    private ISxLossTimeMonthlyService lossTimeMonthlyService;

    @PostMapping(value = "/sx/ext/loss-time-monthly/query")
    @ApiOperation(value = "月度非作业汇总（损失时间）报表")
    public ResultBean queryLossTimeMonthly(@RequestBody SxLossTimeMonthlyModel data) {
        try {
            ValidatorBean.beginValid(data)
                    .notNull(MesExtConstWords.DIMENSION, data.getDimension())
                    .notNull(MesExtConstWords.WORK_TIME, data.getWorkTime())
                    .notNull(MesExtConstWords.CATEGORY_CODE, data.getCategoryCode())
                    .notNull(MesExtConstWords.CATEGORY_NAME, data.getCategoryName());
            data.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(lossTimeMonthlyService.queryLossTimeMonthly(data));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/sx/ext/loss-time-monthly/download")
    @ApiOperation(value = "月度非作业汇总（损失时间）报表导出")
    public ResultBean doDownLoadLossTimeMonthly(HttpServletResponse resp, @RequestBody SxLossTimeMonthlyModel data) {
        try {
            ValidatorBean.beginValid(data)
                    .notNull(MesExtConstWords.DIMENSION, data.getDimension())
                    .notNull(MesExtConstWords.WORK_TIME, data.getWorkTime())
                    .notNull(MesExtConstWords.CATEGORY_CODE, data.getCategoryCode())
                    .notNull(MesExtConstWords.CATEGORY_NAME, data.getCategoryName());
            data.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            lossTimeMonthlyService.doDownLoadLossTimeMonthly(resp, data);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
