package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel.bu;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtEnumUtil;
import cn.estsh.i3plus.mes.apiservice.util.MesCommonUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesContainer;
import cn.estsh.i3plus.pojo.mes.repository.MesContainerRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesContainerTypeRepository;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description : 容器导入，支持编辑
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2019-04-23
 * @Modify:
 **/
@Service
public class BuContainerExcelService implements IExcelImportExtService {

    @Autowired
    private MesContainerRepository containerRepository;

    @Autowired
    private MesContainerTypeRepository containerTypeRepository;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 容器导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractContainer(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelContainer(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 容器导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractContainer(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesContainer
        List<MesContainer> containerList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的容器代码唯一校验，key：ctNo，value：row
        Map<String, String> ctNoMap = new HashMap<>();

        // 容器类型表
        Map<String, Object> ctCodeEntityMap;
        try {
            ctCodeEntityMap = MesCommonUtil.getCodeEntityMap(containerTypeRepository, MesExtConstWords.CT_CODE, organizeCode, "容器类型表");
        } catch (Exception e) {
            ctCodeEntityMap = null;
        }

        //查询容器表
        List<MesContainer> containers = containerRepository.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});
        Map<String, List<MesContainer>> containerMap = CollectionUtils.isEmpty(containers) ? null :
                containers.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesContainer::getCtNo));

        //从excel表的第5行数据开始导入，getFirstRowNum是从0行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //容器代码
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                String ctNo = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //容器类型
                String ctCode = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    ctCode = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                boolean isExist = false;
                //校验容器代码不重复
                if (!ctNoMap.containsKey(ctNo)) {
                    ctNoMap.put(ctNo, String.valueOf(rowNum));
                    //校验 容器代码 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(containerMap) && containerMap.containsKey(ctNo)) {
                        isExist = true;
                    }
                } else {
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据跟第" + ctNoMap.get(ctNo) + "行的第A列数据重复;";
                }

                //校验容器类型
                if (StringUtils.isEmpty(ctCode)) {
                    errorNum ++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                } else {
                    if (CollectionUtils.isEmpty(ctCodeEntityMap) || !ctCodeEntityMap.containsKey(ctCode)) {
                        errorNum ++;
                        cellNum += "B;";
                        errorInfo += "第B列数据无效;";
                    }
                }

                //没有错误的时候，封装MesContainer
                if (errorNum == 0) {
                    MesContainer container;
                    if(isExist){
                        container = containerMap.get(ctNo).get(0);
                        if(container.getUseStatus() == MesExtEnumUtil.CONTAINER_USE_STATUS.LEISURE.getValue()){
                            container.setCtCode(ctCode);
                        }
                        ConvertBean.serviceModelUpdate(container, userName);
                    }else{
                        container = new MesContainer();
                        container.setUseStatus(MesExtEnumUtil.CONTAINER_USE_STATUS.LEISURE.getValue());
                        container.setCtCode(ctCode);
                        ConvertBean.serviceModelInitialize(container, userName);
                    }
                    container.setCtNo(ctNo);
                    container.setSyncStatus(MesExtEnumUtil.IF_SYNC_STATUS.NO_SYNC.getValue());
                    container.setOrganizeCode(organizeCode);

                    containerList.add(container);

                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultModel.setExcelList((failRowNum > 0) ? null : containerList);
        return excelImportResultModel;
    }

    /**
     * 容器导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelContainer(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesContainer> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) {
                for (MesContainer container : excelList) {
                    if(StringUtils.isEmpty(container.getId())){
                        containerRepository.insert(container);
                    }else{
                        containerRepository.save(container);
                    }
                }
            }
        }
    }
}
