package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel.jx;

import cn.estsh.i3plus.ext.mes.api.base.IExcelImportExtService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.CommonExtService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesBadnessType;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportErrorExtModel;
import cn.estsh.i3plus.ext.mes.pojo.model.ExcelImportResultExtModel;
import cn.estsh.i3plus.ext.mes.pojo.repository.MesBadnessTypeRepository;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:不良类型导入
 * @Reference:
 * @Author: wangjie
 * @CreateDate: 2019\11\4 10:39
 * @Modify:
 **/
@Service
public class JxBadnessTypeExcelService implements IExcelImportExtService {

    @Autowired
    private MesBadnessTypeRepository buBadnessTypeDao;

    @Autowired
    private CommonExtService commonExtService;

    /**
     * 不良类型导入
     * @param workbook
     * @param organizeCode
     * @param userName
     * @return
     */
    @Override
    public ExcelImportResultExtModel insertDataByExcel(Workbook workbook, String organizeCode, String userName) {
        //读取表格
        ExcelImportResultExtModel excelImportResultExtModel = this.sheetExtractBadnessType(workbook.getSheetAt(0), organizeCode, userName);
        //数据入库
        this.insertExcelBadnessType(excelImportResultExtModel);
        return excelImportResultExtModel;
    }

    /**
     * 不良类型导入-读取表格
     * @param sheetAt
     * @param organizeCode
     * @param userName
     * @return
     */
    private ExcelImportResultExtModel sheetExtractBadnessType(Sheet sheetAt, String organizeCode, String userName) {
        //从0行开始读取
        int totalNumberOfRows = sheetAt.getLastRowNum() + 1;

        //MesBadnessType集合
        List<MesBadnessType> sxQmsBuBadnessTypeList = new ArrayList<>();

        //成功数量
        Integer successRowNum = 0;
        //失败数量
        Integer failRowNum = 0;
        //错误的行号
        String errorRows = "";

        //错误行信息集合
        List<ExcelImportErrorExtModel> excelImportErrorExtModels = new ArrayList<>();

        //用于EXCEL表中的 不良类型代码 唯一校验，key：badnessTypeCode，value：row
        Map<String,String> existMap = new HashMap<>();

        List<MesBadnessType> sxQmsBuBadnessTypeDbs = buBadnessTypeDao.findByProperty(
                new String[]{MesExtConstWords.ORGANIZE_CODE, MesExtConstWords.IS_DELETED},
                new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()});

        Map<String, List<MesBadnessType>> badnessTypeMap = CollectionUtils.isEmpty(sxQmsBuBadnessTypeDbs) ? null :
                sxQmsBuBadnessTypeDbs.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getBadnessTypeCode())))
                .collect(Collectors.groupingBy(MesBadnessType::getBadnessTypeCode));

        //从excel表的第5行数据开始导入，getFirstRowNum是从5行开始读取
        for (int i = (sheetAt.getFirstRowNum() + 4); i < totalNumberOfRows; i ++) {
            Row row = sheetAt.getRow(i);
            //空行跳过
            if (null == row) {
                continue;
            }
            //获取总列数
            Short lastCellNum = row.getLastCellNum();
            if (lastCellNum > 0) {

                int rowNum = i + 1;    //当前行号
                int errorNum = 0;      //错误数量
                String cellNum = "";   //错误列号
                String errorInfo = ""; //错误信息

                //检验顺序
                try {
                    //第一列必须有值，否则跳过
                    row.getCell(0).setCellType(CellType.STRING);
                } catch (Exception e) {
                    continue;
                }
                //不良类型代码
                String badnessTypeCode = row.getCell(0, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                //不良类型名称
                String badnessTypeName = null;
                try {
                    row.getCell(1).setCellType(CellType.STRING);
                    badnessTypeName = row.getCell(1, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }
                //颜色
                String color = null;
                try {
                    row.getCell(2).setCellType(CellType.STRING);
                    color = row.getCell(2, Row.MissingCellPolicy.CREATE_NULL_AS_BLANK).getStringCellValue().trim();
                } catch (Exception e) {
                }

                if (StringUtils.isEmpty(badnessTypeCode)) {
                    errorNum++;
                    cellNum += "A;";
                    errorInfo += "第A列数据必填;";
                }

                if (StringUtils.isEmpty(badnessTypeName)) {
                    errorNum++;
                    cellNum += "B;";
                    errorInfo += "第B列数据必填;";
                }

                if (StringUtils.isEmpty(color)) {
                    errorNum++;
                    cellNum += "C;";
                    errorInfo += "第C列数据必填;";
                }

                boolean isExist = false;
                if(!existMap.containsKey(badnessTypeCode)){

                    existMap.put(badnessTypeCode, String.valueOf(rowNum));
                    //校验 不良类型数据 是否已经存在数据库
                    if (StringUtils.isEmpty(errorInfo) && !CollectionUtils.isEmpty(badnessTypeMap) && badnessTypeMap.containsKey(badnessTypeCode)) {
                        isExist = true;
                    }
                }else {
                    errorNum ++;
                    cellNum += "A;";
                    errorInfo += "第A列数据跟第" + existMap.get(badnessTypeCode) + "行的第A列数据重复;";
                }

                //没有错误的时候，封装SxQmsOqcProject
                if (errorNum == 0) {
                    MesBadnessType item;
                    if(isExist) {
                        item = badnessTypeMap.get(badnessTypeCode).get(0);
                        ConvertBean.serviceModelUpdate(item, userName);
                    }else{
                        item = new MesBadnessType();
                    }

                    item.setBadnessTypeCode(badnessTypeCode);
                    item.setBadnessTypeName(badnessTypeName);
                    item.setColor(color);
                    item.setOrganizeCode(organizeCode);
                    ConvertBean.serviceModelInitialize(item, userName);
                    sxQmsBuBadnessTypeList.add(item);
                    successRowNum ++;
                } else {
                    //封装错误行信息ExcelImportErrorExtModel
                    excelImportErrorExtModels = commonExtService.getExcelImportErrorModels(excelImportErrorExtModels, rowNum, errorNum, cellNum, errorInfo);

                    errorRows += rowNum + ";";
                    failRowNum ++;
                }
            }
        }

        //校验EXCEL数据
        commonExtService.checkExcelData(failRowNum, successRowNum, errorRows);

        //封装返回结果
        ExcelImportResultExtModel excelImportResultExtModel = commonExtService.getExcelImportResultModel(failRowNum, successRowNum, excelImportErrorExtModels, errorRows);
        excelImportResultExtModel.setExcelList((failRowNum > 0) ? null : sxQmsBuBadnessTypeList);
        return excelImportResultExtModel;
    }

    /**
     * 不良类型导入-数据入库
     * @param excelImportResultExtModel
     */
    private void insertExcelBadnessType(ExcelImportResultExtModel excelImportResultExtModel) {
        //导入数据
        if (null != excelImportResultExtModel) {
            List<MesBadnessType> excelList = excelImportResultExtModel.getExcelList();
            if (!CollectionUtils.isEmpty(excelList)) buBadnessTypeDao.saveAll(excelList);
        }
    }
}
